package com.digiwin.athena.app.infra.common.utils;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.log4j.Log4j;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Author: xieps
 * @Date: 2023/3/21 13:49
 * @Version 1.0
 * @Description
 */
@Log4j
public class TransferUtil {


    public static final ObjectMapper objectMapper = new ObjectMapper();


    public static <T> List<T> string2List(String messageBody, Class<T> clazz, String key) {
        // 获取请求参数
        JSONObject parameter = TransferUtil.getAthenaParameter(messageBody);
        return TransferUtil.convertString2List(parameter.toJSONString(), key, clazz);
    }

    /**
     * String 转model List
     * @param param
     * @param key
     * @param clazz
     * @param <T>
     * @return
     */
    public static  <T> List<T> convertString2List(String param, String key, Class<T> clazz) {
        Map<String, Object> paramMap = str2Map(param);
        return getMapTs(paramMap, key, clazz);
    }




    /**
     * String 转 map
     *
     * @param param
     * @return
     * @throws Exception
     */
    public static Map<String, Object> str2Map(String param) {
        if (StringUtils.isEmpty(param)) {
            return new HashMap<>();
        }
        try {
            return objectMapper.readValue(param, new TypeReference<Map<String, Object>>() {
            });
        } catch (IOException e) {
            log.error(e.getMessage());
            return new HashMap<>();
        }
    }


    /**
     * map 取 model list
     *
     * @param map
     * @return
     * @throws Exception
     */
    public static <T> List<T> getMapTs(Map map, String key, Class<T> clazz) {
        if (!MapUtils.isEmpty(map)) {
            Object obj = map.get(key);
            if (obj != null) {
                ObjectMapper mapper = new ObjectMapper();
                try {
                    String objStr = mapper.writeValueAsString(obj);
                    JavaType type = mapper.getTypeFactory().constructCollectionType(List.class, clazz);
                    return mapper.readValue(objStr, type);
                } catch (Exception ex) {
                    log.error(ex.getMessage());
                    return new ArrayList<>();
                }
            }
        }
        return new ArrayList<>();
    }


    public static JSONObject getAthenaParameter(String messageBody){
        JSONObject stdData = JSONObject.parseObject(messageBody).getJSONObject("std_data");
        JSONObject parameter = stdData.getJSONObject("parameter");
        return parameter;
    }

}
