package com.digiwin.athena.app.service.staff;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.digiwin.app.container.exceptions.DWRuntimeException;
import com.digiwin.app.service.DWEAIResult;
import com.digiwin.athena.app.infra.entity.StaffEntity;
import com.digiwin.athena.app.infra.repository.StaffRepository;
import com.digiwin.athena.app.infra.service.StaffService;
import com.digiwin.athena.opt.common.eai.EAIRequest;
import com.digiwin.athena.opt.common.eai.EAIUtil;
import com.digiwin.athena.opt.common.eai.service.AbsEAIService;
import lombok.extern.log4j.Log4j2;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Auther: zhenggl
 * @Date: 2023/4/28
 */
@Service
@Log4j2
public class StaffCreateEAIService extends AbsEAIService {

    @Resource
    private StaffRepository staffRepository;

    @Resource
    private StaffService staffService;

    @Override
    public String getServiceName() {
        return StaffUtil.DEMO_WAIT_REGISTER_PERSONNEL_INFO_CREATE;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public DWEAIResult execute(Map<String, String> headers, String messageBody) throws Exception {
        //反序列化字段
        EAIRequest request = new EAIRequest(messageBody);
        List<StaffEntity> staffList = request.getObject("wait_register_personnel", StaffUtil.LIST_ENTITY_STAFF);
        if (CollectionUtils.isEmpty(staffList)){
            throw new DWRuntimeException("缺少必要参数");
        }
        //判断编号跟邮箱是否存在
        LambdaQueryWrapper<StaffEntity> lmq = new LambdaQueryWrapper<>();
        List<String> employeeNoList = staffList.stream().map(StaffEntity::getEmployeeNo).collect(Collectors.toList());
        List<String> emailList = staffList.stream().map(StaffEntity::getEmail).collect(Collectors.toList());
        lmq.in(StaffEntity::getEmployeeNo,employeeNoList);
        lmq.or().in(StaffEntity::getEmail,emailList);
        List<StaffEntity> staffEntities = staffRepository.selectList(lmq);
        if (!CollectionUtils.isEmpty(staffEntities)){
            throw new DWRuntimeException("员工编号或邮箱重复");
        }
        staffService.saveBatch(staffList);
        List<StaffEntity> collect = staffList.stream().map(item -> new StaffEntity(item.getId(),item.getEmployeeNo())).collect(Collectors.toList());
        return EAIUtil.buildEAIResult(new JSONObject().fluentPut("wait_register_personnel",collect));
    }
}
